/*
 *
 * Copyright(C) OASIS(R) 2010. All Rights Reserved.
 * OASIS trademark, IPR and other policies apply.
 *
 */

/**
 * C++ component implementation with
 * 1 service with interface Service1,
 * 1 reference (1..1) with interface Service47 which has a Callback
 * interface which is implemented by this class
 * reference gets called when service1 operation1 is invoked
 * @author Bryan Aupperle
 *
 */

#include "Service1Callback47Impl.h"
#include "oasis/sca/RefCountingPointer.h"
#include "oasis/sca/ComponentContext.h"
#include "commonj/sdo/sdo.h"
#include <unistd.h>
using namespace oasis::sca;

// Construction/Destruction

Service1Callback47Impl::Service1Callback47Impl()
{
	ComponentContextPtr context = ComponentContext::getCurrent();

	commonj::sdo::DataObjectPtr properties = context->getProperties();

	serviceName = properties->getString("serviceName");

	return;
}


Service1Callback47Impl::~Service1Callback47Impl()
{
	return;
}

// Implementation

std::string Service1Callback47Impl::operation1(const std::string input) {
	std::string result = "";

	ComponentContextPtr context = ComponentContext::getCurrent();

	ServiceProxyPtr service = context->getService("reference1");
	reference1 = dynamicCast<Service47Proxy>(service);

	if (reference1) {
		Service47Proxy::operation1ResponsePtr response = reference1->operation1Async(input);
		while (!response->isReady())
			sleep(2);

		// The response is ready and can be accessed without blocking.
		result = response->getReturn();
	}

	return serviceName + " operation1 invoked" + " " + result;
}

std::string Service1Callback47Impl::callback1(const std::string input) {
	sleep(10);
	return serviceName + " callback1 invoked";
}
