/*
 *
 * Copyright(C) OASIS(R) 2010. All Rights Reserved.
 * OASIS trademark, IPR and other policies apply.
 *
 */

/**
 * C++ component implementation with
 * 1 service with interface Service1,
 * 1 reference (1..1) with interface Service46
 * reference gets called when service1 operation1 is invoked
 * @author Bryan Aupperle
 *
 */

#include "Service1Impl53.h"
#include "oasis/sca/RefCountingPointer.h"
#include "oasis/sca/ComponentContext.h"
#include "commonj/sdo/sdo.h"
#include <unistd.h>

using namespace oasis::sca;

// Construction/Destruction

Service1Impl53::Service1Impl53()
{
	ComponentContextPtr context = ComponentContext::getCurrent();

	commonj::sdo::DataObjectPtr properties = context->getProperties();

	serviceName = properties->getString("serviceName");

	return;
}


Service1Impl53::~Service1Impl53()
{
	return;
}

// Implementation

class operation1Callback : public Service46Proxy::operation1Callback {
public:
	std::string asyncRsp;

	void invoke(Service46Proxy::operation1ResponsePtr response) {
		asyncRsp = response->getReturn();
	}
};

typedef RefCountingPointer<operation1Callback> operation1CallbackPtr;

std::string Service1Impl53::operation1(const std::string input) {
	std::string result;

	ComponentContextPtr context = ComponentContext::getCurrent();

	ServiceProxyPtr service = context->getService("reference1");
	reference1 = dynamicCast<Service46Proxy>(service);

	if (reference1) {
		operation1CallbackPtr callback = new operation1Callback();
		Service46Proxy::operation1ResponsePtr response = reference1->operation1Async(input);
		response->setCallback(callback);
		sleep(15);
		result = callback->asyncRsp;
	}

	return serviceName + " operation1 invoked" + " " + result;
}
