/*
 *
 * Copyright(C) OASIS(R) 2010. All Rights Reserved.
 * OASIS trademark, IPR and other policies apply.
 *
 */

/**
 * C++ component implementation with
 * 1 service with interface Service1,
 * 1 reference (1..1) with interface Service46
 * reference gets called when service1 operation1 is invoked
 * @author Bryan Aupperle
 *
 */

#include "Service1Impl54.h"
#include "oasis/sca/RefCountingPointer.h"
#include "oasis/sca/ComponentContext.h"
#include "commonj/sdo/sdo.h"
#include <unistd.h>

using namespace oasis::sca;

// Construction/Destruction

Service1Impl54::Service1Impl54()
{
	ComponentContextPtr context = ComponentContext::getCurrent();

	commonj::sdo::DataObjectPtr properties = context->getProperties();

	serviceName = properties->getString("serviceName");

	return;
}


Service1Impl54::~Service1Impl54()
{
	return;
}

// Implementation


std::string Service1Impl54::operation1(const std::string input) {
	std::string result;

	ComponentContextPtr context = ComponentContext::getCurrent();

	ServiceProxyPtr service = context->getService("reference1");
	reference1 = dynamicCast<Service46Proxy>(service);

	if (reference1) {
		Service46Proxy::operation1ResponsePtr response = reference1->operation1Async(input);
		while (!response->isReady())
			sleep(2);

		// The response is ready and can be accessed without blocking.
		result = response->getReturn();
	}

	return serviceName + " operation1 invoked" + " " + result;
}
