/*
 *
 * Copyright(C) OASIS(R) 2010. All Rights Reserved.
 * OASIS trademark, IPR and other policies apply.
 *
 */

/**
 * C++ component implementation with
 * 1 service with interface Service1,
 * 1 reference (1..1) with interface Service1
 * Two threads are created and reference gets called on each thread when
 * service1 operation1 is invoked
 * @author Bryan Aupperle
 *
 */

#include "Service1Impl56.h"
#include "oasis/sca/RefCountingPointer.h"
#include "oasis/sca/ComponentContext.h"
#include "commonj/sdo/sdo.h"
#include <pthread.h>

using namespace oasis::sca;

// Construction/Destruction

Service1Impl56::Service1Impl56()
{
	ComponentContextPtr context = ComponentContext::getCurrent();

	commonj::sdo::DataObjectPtr properties = context->getProperties();

	serviceName = properties->getString("serviceName");

	return;
}


Service1Impl56::~Service1Impl56()
{
	return;
}

// Implementation


void* thread1(void *param) {
	std::string result;

	Service1Impl56 *obj=(Service1Impl56*)param;
	result = obj->reference1->operation1("a");
	strcpy(obj->resultT, result.c_str());

	pthread_exit((void *)obj->resultT);
};

void* thread2(void *param) {
	std::string result;

	Service1Impl56 *obj=(Service1Impl56*)param;
	result = obj->reference1->operation1("b");

	pthread_exit(NULL);
};

std::string Service1Impl56::operation1(const std::string input) {
	void * result;
	pthread_t tid1, tid2;

	ComponentContextPtr context = ComponentContext::getCurrent();

	ServiceProxyPtr service = context->getService("reference1");
	reference1 = dynamicCast<Service1Proxy>(service);

	if (reference1)
		pthread_create(&tid1, NULL, thread1, (void*)this);
		pthread_create(&tid2, NULL, thread2, (void*)this);

		pthread_join(tid1, &result);
		pthread_join(tid2, NULL);

	return serviceName + " operation1 invoked" + " " + (char *)result;
}
