/*
 *
 * Copyright(C) OASIS(R) 2010. All Rights Reserved.
 * OASIS trademark, IPR and other policies apply.
 *
 */

/**
 * A test service interface
 * @author Bryan Aupperle
 *
 */

#include <string>
#include <wchar.h>
#include <stdint.h>
#include <time.h>
#include "commonj/sdo/sdo.h"

// @Remotable
class Service21 {
public:
	/**
	 * Member function for invoking testcase service
	 * @param input - input parameter(s) as a std::string
	 * @return - output data as a std::string
	 */
	virtual std::string operation1(const std::string input) const = 0;

	/**
	 * Member function for invoking testcase service
	 * @param a - int parameter
	 * @param b - long parameter
	 * @param c - long long parameter
	 * @return - output data as a short
	 */
	virtual short operation2(int a, long b, long long c) const = 0;

	/**
	 * Member function for invoking testcase service
	 * @param a - unsigned int parameter
	 * @param b - unsigned long parameter
	 * @param c - unsigned long long parameter
	 * @return - output data as a unsigned short
	 */
	virtual unsigned short operation3(unsigned int a, unsigned long b, unsigned long long c) const = 0;

	/**
	 * Member function for invoking testcase service
	 * @param a - signed char parameter
	 * @param b - char parameter
	 * @return - output data as a unsigned char
	 */
	virtual unsigned char operation4(signed char a, char b) const = 0;

	/**
	 * Member function for invoking testcase service
	 * @param a - double parameter
	 * @param b - long double parameter
	 * @return - output data as a float
	 */
	virtual float operation5(double a, long double b) const = 0;

	/**
	 * Member function for invoking testcase service
	 * @param a - int16_t parameter
	 * @param b - int32_t parameter
	 * @param c - int64_t  parameter
	 * @return - output data as a int8_t
	 */
	virtual int8_t operation6(int16_t a, int32_t b, int64_t c) const = 0;

	/**
	 * Member function for invoking testcase service
	 * @param a - uint16_t parameter
	 * @param b - uint32_t parameter
	 * @param c - uint64_t  parameter
	 * @return - output data as a uint8_t
	 */
	virtual uint8_t operation7(uint16_t a, uint32_t b, uint64_t c) const = 0;

	/**
	 * Member function for invoking testcase service
	 * @param a - wide char parameter
	 * @param b - DataObject parameter
	 * @param c - char * parameter
	 * @return - output data as a wide char string
	 */
	virtual wchar_t *operation8(wchar_t a, commonj::sdo::DataObjectPtr b, const char *c) const = 0;

	/**
	 * Member function for invoking testcase service
	 * @param a - time_t parameter
	 * @param b - struct tm parameter
	 * @return - output data as a _Bool
	 */
	virtual bool operation9(time_t a,  struct tm b) const = 0;
};
